#!/bin/sh 
apt-get install --no-install-recommends --yes build-essential cmake doxygen git \
    libboost-date-time-dev libboost-dev libboost-filesystem-dev \
    libboost-graph-dev libboost-iostreams-dev libboost-program-options-dev \
    libboost-python-dev libboost-regex-dev libboost-serialization-dev \
    libboost-thread-dev libcoin-dev libeigen3-dev libgtkglext1-dev \
    libgts-dev libkdtree++-dev libkml-dev libmedc-dev libocct-data-exchange-dev \
    libocct-draw-dev libocct-foundation-dev libocct-modeling-algorithms-dev \
    libocct-modeling-data-dev libocct-ocaf-dev libocct-visualization-dev \
    libopencv-dev libproj-dev libpyside2-dev libqt5svg5-dev libqt5webkit5-dev \
    libqt5xmlpatterns5-dev libshiboken2-dev libvtk9-dev libvtk9-qt-dev \
    libvtk-dicom-dev libx11-dev libxerces-c-dev libxmu-dev libxmuu-dev \
    libzipios++-dev netgen netgen-headers pyside2-tools python3-dev \
    python3-matplotlib python3-pivy python3-ply python3-pyside2.qtsvg \
    python3-pyside2.qtuitools qtchooser qttools5-dev shiboken2 swig
